import React, { memo } from 'react';
import { clamp, DashPathEffect, Group, interpolate, Line, Text, useComputedValue, useValue, useValueEffect, vec } from '@shopify/react-native-skia';
import dayjs from 'dayjs';
import { convertDataArrToObj } from './helpers';
import Tooltip from './Tooltip';
import { CHART_TOOLTIP_MARGIN } from './constants';
const LineChartTooltip = _ref => {
  let {
    data,
    x,
    xScaleBounds,
    chartHeight,
    font,
    backgroundColor = 'black',
    width = 100,
    height = 50,
    dateFormat = 'MMM D',
    startDate,
    endDate
  } = _ref;
  const dataMap = convertDataArrToObj(data, dateFormat);
  const tooltipContentDate = useValue('');
  const tooltipContentValue = useValue('');
  const transformLine = useComputedValue(() => {
    return [{
      translateX: clamp(x.current, xScaleBounds[0], xScaleBounds[1])
    }];
  }, [x]);
  const transformTooltip = useComputedValue(() => {
    const xPos = clamp(x.current, xScaleBounds[0], xScaleBounds[1]);
    const isLeftPos = xPos + width >= xScaleBounds[1];
    const translateX = isLeftPos ? xPos - width - CHART_TOOLTIP_MARGIN : xPos + CHART_TOOLTIP_MARGIN;
    return [{
      translateX
    }];
  }, [x]);
  const tooltipInterpolatedXPosition = useComputedValue(() => {
    return interpolate(x.current, xScaleBounds, [dayjs(startDate).valueOf(), dayjs(endDate).valueOf()]);
  }, [x, startDate, endDate]);

  // todo find a better way to calc contentDate
  useValueEffect(tooltipInterpolatedXPosition, () => {
    const contentDate = dayjs(tooltipInterpolatedXPosition.current).format(dateFormat);
    const contentValue = dataMap[contentDate];
    if (contentDate) tooltipContentDate.current = contentDate.toString();
    if (contentValue) tooltipContentValue.current = contentValue.toString();
  });
  return /*#__PURE__*/React.createElement(Group, null, /*#__PURE__*/React.createElement(Group, {
    transform: transformLine
  }, /*#__PURE__*/React.createElement(Line, {
    p1: vec(0, 0),
    p2: vec(0, chartHeight),
    color: "lightblue",
    style: "stroke",
    strokeWidth: 1
  }, /*#__PURE__*/React.createElement(DashPathEffect, {
    intervals: [4, 4]
  }))), /*#__PURE__*/React.createElement(Group, {
    transform: transformTooltip
  }, /*#__PURE__*/React.createElement(Tooltip, {
    backgroundColor: backgroundColor,
    width: width,
    height: height
  }, font ? /*#__PURE__*/React.createElement(Group, {
    color: "white"
  }, /*#__PURE__*/React.createElement(Text, {
    font: font,
    x: 10,
    y: 20,
    text: tooltipContentDate
  }), /*#__PURE__*/React.createElement(Text, {
    font: font,
    x: 10,
    y: 40,
    text: tooltipContentValue
  })) : null)));
};
export default /*#__PURE__*/memo(LineChartTooltip);
//# sourceMappingURL=LineChartTooltip.js.map