import { useMemo } from 'react';
import { Gesture } from 'react-native-gesture-handler';
import { useSharedValue } from 'react-native-reanimated';
export function usePanGesture(_ref) {
  let {
    holdDuration = 300,
    xScaleBounds = [0, 1]
  } = _ref;
  const x = useSharedValue(xScaleBounds[0]);
  const y = useSharedValue(0);
  const isPanGestureActive = useSharedValue(false);
  const panGesture = useMemo(() => Gesture.Pan().activateAfterLongPress(holdDuration).onChange(e => {
    // clamp x value to xScaleBounds.
    // can't move it to a separate function as ...
    //  ...it doesn't work with shared values.
    x.value = Math.min(Math.max(xScaleBounds[0], e.x), xScaleBounds[1]);
    y.value = e.y;
  }).onStart(() => {
    isPanGestureActive.value = true;
  }).onEnd(() => {
    isPanGestureActive.value = false;
  }), [holdDuration, isPanGestureActive, x, y, xScaleBounds]);
  return useMemo(() => ({
    gesture: panGesture,
    isActive: isPanGestureActive,
    x,
    y
  }), [isPanGestureActive, panGesture, x, y]);
}
//# sourceMappingURL=usePanGesture.js.map